==> pyraptor-0.1.0.tar.gz: pyraptor-0.1.0/pyraptor/__init__.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:        pyraptor.py (Python 3.x).
# description: A tool for searching, extracting, debugging and displaying
#              information from multiple text files.
# author:      Antonio Suárez Jiménez, pherkad13@gmail.com
# date:        23-07-2015
#
#--------------------------------------------------------------------

'''A tool for searching, extracting, debugging and displaying information
from multiple text files.
'''

__author__ = 'Antonio Suárez Jiménez, pherkad13@gmail.com'
__title__= 'pyraptor'
__date__ = '2015-07-23'
__version__ = '0.1.0'
__license__ = 'GNU GPLv3'

import os, configparser, collections

class Pyraptor(object):
	'''Pyraptor for extracting and displaying information from a set of files 
	of the same type; and creating a single CSV file with all the selected
	information.
	
	The information in the files may be in multiple rows:
	
	PC01.txt:
	User=ms123
	Name=Mayra Sanz
	SO=GNU/Linux
	IP=10.226.140.1
	
	But, also, the information may be in several columns. It is possible
	to read data from multiple fields in a single line:
	
	PC01.log:
	User: ms123	    Name: Mayra Sanz
	SO: GNU/Linux   IP: 10.226.140.1		

	Example: data from the following files...

	PC01.txt:
	User=ms123
	Name=Mayra Sanz
	OS=GNU/Linux
	IP=10.226.140.1

	PC02.txt:
	User=lt001
	Name=Luis Toribio
	OS=GNU/Linux
	IP=10.226.140.2

	PC03.txt:
	User=co205
	Name=Clara Osto
	OS=Win
	IP=10.226.140.3

	... You can create a CSV file with the following information:

	User,Name,OS,IP
	MS123,Mayra Sanz,GNU/linux,10.226.140.1
	LT001,Luis Toribio,GNU/linux,10.226.140.2
	CO205,Clara Osto,Win,10.226.140.3

	To achieve this you need to create a template (.rap) with Pyraptor, 
	which is similar to an INI file with the following information:

	users.rap:
	[General]
	description = Get list of users
	extension = txt
	prefix = PC
	output_folder = txt
	input_folder = txt
	output_file = users.csv
	delimiter = ,
	quotechar = "
	include_header = 1
	include_file = 0
	search_multiple = 0

	[Fields]
	user = User=
	name = Name=
	os = OS=
	ip = IP=

	[Rules]
	rule1 = ('user', 'UPPER')

	To create .rap template (If the .rap template exists it is not saved):

	from pyraptor import Pyraptor
	rap1 = Pyraptor()		
	rap1.description = 'Get list of users'
	rap1.extension = 'txt'
	rap1.prefix = 'PC'
	rap1.input_folder = 'txt'
	rap1.output_folder = 'txt'
	rap1.output_file = 'users.csv'
	rap1.delimiter = ','
	rap1.quotechar = '"'
	rap1.include_header = '1'
	rap1.include_file = '0'
	rap1.search_multiple = '0'
	rap1.fields['user'] = 'User='
	rap1.fields['name'] = 'Name='
	rap1.fields['os'] = 'OS='
	rap1.fields['ip'] = 'IP='
	rap1.rules.append(('user', 'UPPER'))
	rap1.Save("users.rap") 	
	del rap1

	Opens template (.rap) and creates (.csv) file from the data read from
	multiple text files:

	from pyraptor import Pyraptor
	import csv
	rap2 = Pyraptor()
	rap2.Open('users.rap')
	if rap2.number_errors == 0:			
		file_csv = open(rap2.output_file, 'w', newline='')
		csv_output = csv.writer(file_csv, 
					delimiter=rap2.delimiter,
					quotechar=rap2.quotechar, 
					quoting=csv.QUOTE_MINIMAL)
		if rap2.include_header == '1':
			fields_list = rap2.BuildHeader()
			print(fields_list)
			csv_output.writerow(fields_list)
							
		for row in rap2.list_files:
			new_record = rap2.ApplyRules(rap2.BuildRow(row))			
			new_record = list(new_record.values())
			print(new_record)
			csv_output.writerow(new_record)
			print('File 'users.csv' saved.')			
		file_csv.close()			
	else:
		print(rap2.ShowError())
	del rap2

	'''	
	app_folder = os.getcwd() + '/'
	def __init__(self):
		'''Initialize a .rap template
		
		Attribute List:
		
		- description: short descripton of .rap template
		- extension: extension of the files to read
		- prefix: files must begin with this string
		- input_folder: folder of files to read
		- output_folder: output folder to save file with result
		- output_file: output filename
		- delimiter: delimiter character
		- quotechar: quote character
		- include_header: '0' or '1'
		- include_file: '0' or '1'
		- search_multiple: '0' or '1'
		- fields: dictionary with fieldnames and search string (read template)
		- record: dictionary with fieldnames and values (read template)
		- rules: list of rules (read template)
		- list_files: list of filenames to read (auto)
		- errors: list of errors (auto)
		- number_errors: number of errors after you open or save a template
		
		>>> from pyraptor import Pyraptor		
		>>> rap1 = Pyraptor()
		>>> rap1.description = 'Description'
		>>> ...				
		'''		
		self.description = ''
		self.extension = 'txt'
		self.prefix = ''
		self.input_folder = Pyraptor.app_folder
		self.output_folder = Pyraptor.app_folder
		self.output_file = 'output.csv'
		self.delimiter = ','
		self.quotechar = '"'
		self.include_header = '1'
		self.include_file = '0'
		self.search_multiple = '0'
		self.fields = collections.OrderedDict({})
		self.record = collections.OrderedDict({})
		self.rules = []
		self.list_files = []
		self.errors = []
		self.number_errors = 0
			
	def Open(self, name):
		'''Open .rap template
		
		name -- .rap template name
		
		>>> rap1.Open('template.rap')
		'''
		self.errors = []		
		if os.path.exists(name):
			config_file = configparser.ConfigParser()
			config_file.read(name)
			if config_file.has_option('General','description'):
				self.description = config_file['General']['description']
			else:
				self.description = ''
			if config_file.has_option('General','extension'):
				self.extension = config_file['General']['extension']
			else:
				self.extension = "txt"
			if config_file.has_option('General','prefix'):
				self.prefix = config_file['General']['prefix']
			else:
				self.prefix = ''
			if config_file.has_option('General','input_folder') and \
				os.path.exists(config_file['General']['input_folder']):
				self.input_folder = config_file['General']['input_folder'] + '/'
			else:
				self.input_folder = Pyraptor.app_folder
				self.errors.append('No source folder. ')
			if config_file.has_option('General','output_folder') and \
				os.path.exists(config_file['General']['output_folder']):
				self.output_folder = config_file['General']['output_folder'] + '/'
			else:
				self.output_folder = Pyraptor.app_folder
				self.errors.append('No target folder. ')
			if config_file.has_option('General','output_file') and \
				config_file['General']['output_file'] != '':
				self.output_file = self.output_folder + config_file['General']['output_file']
			else:
				self.output_file = self.output_folder + "output.csv"				
			if config_file.has_option('General','delimiter') and \
				config_file['General']['delimiter'] != '':
				self.delimiter = config_file['General']['delimiter']
			else:
				self.delimiter = ','
			if config_file.has_option('General','quotechar') and \
				config_file['General']['quotechar'] != '':
				self.quotechar = config_file['General']['quotechar']
			else:
				self.quotechar = '"'
			if config_file.has_option('General','include_header') and \
				config_file['General']['include_header'] in ['0','1'] :
				self.include_header = config_file['General']['include_header']
			else:
				self.include_header = '1'
			if config_file.has_option('General','include_file') and \
				config_file['General']['include_file'] in ['0','1']:
				self.include_file = config_file['General']['include_file']
			else:
				self.include_file = '0'
			if config_file.has_option('General','search_multiple') and \
				config_file['General']['search_multiple'] in ['0','1']:
				self.search_multiple = config_file['General']['search_multiple']
			else:
				self.search_multiple = '0'

			self.fields = collections.OrderedDict({})
			self.record = collections.OrderedDict({})
			if 'Fields' in config_file:
				if self.include_file == '1':
					self.fields['f___n'] = ''
					self.record['f___n'] = ''					
				for option, value in config_file['Fields'].items():
					self.fields[option] = value
					self.record[option] = ''
			
			self.rules = []
			if len(self.fields) > 0:			
				if 'Rules' in config_file:	
					for option, value in config_file['Rules'].items():
						if option.upper().startswith('RULE') and len(option) > 4 and \
							option[4:].strip().isnumeric():
								value = eval(value)
								if type(value) is tuple and len(value) > 1:
									self.rules.append(value)
			else:
				self.errors.append('No field/s. ')

		self.list_files = []			
		if len(self.errors) == 0:
			list_file = os.listdir(self.input_folder)
			for actual_file in list_file:
				if actual_file[-3:].upper() == self.extension.upper():
					if actual_file.startswith(self.prefix):
						self.list_files.append(actual_file)
			
			if len(self.list_files) > 0:
				self.list_files.sort()
			else:
				self.errors.append('No file/s to search. ')
		self.number_errors = len(self.errors)

	def Save(self, name):
		'''It saves .rap template
		
		name -- .rap template name
		
		>>> rap1.Save('template.rap')
		'''
		self.errors = []		
		config_file = configparser.ConfigParser()		
		if os.path.exists(name) == False:
			if len(self.fields) > 0:
				config_file['General'] = {}
				config_file['Fields'] = {}
				config_file['Rules'] = {}
				general = config_file['General']
				fields = config_file['Fields']
				rules = config_file['Rules']
				general['description'] = self.description
				general['extension'] = self.extension
				general['prefix'] = self.prefix
				general['output_folder'] = self.output_folder
				general['input_folder'] = self.input_folder
				general['output_file'] = self.output_file			
				general['delimiter'] = self.delimiter
				general['quotechar'] = self.quotechar
				general['include_header'] = self.include_header
				general['include_file'] = self.include_file
				general['search_multiple'] = self.search_multiple
				for option, value in self.fields.items():
					fields[option] = value
				
				for position in range(len(self.rules)):
					rules['rule' + str(position+1)] = str(self.rules[position])
				
				with open(name, 'w') as output_config:
					config_file.write(output_config)
			else:
				self.errors.append('No field/s. ')
		else:
			self.errors.append('The .rap template exists. ')
		self.number_errors = len(self.errors)
	
	def BuildHeader(self):
		'''Buid header
		
		Returns Python list of field names
		
		>>> rap1.BuildHeader()
		'''
		if self.number_errors == 0:
			return(list(self.fields.keys()))
		else:
			return([])

	def BuildRow(self, row):
		'''Buid row (record)
		
		row -- filename to search
		
		Returns Python dictionary with field names and data
		
		>>> rap1.BuildRow('filename')
		'''
		if self.number_errors == 0:
			for option in self.record.keys():
				self.record[option] = ''		
			current_file = open(self.output_folder + row, 'rb')		
			while True:
				current_line = current_file.readline()
				current_line = current_line.decode("utf-8", "ignore")
				if not current_line: break
				current_line = current_line.rstrip()
				len_cl = len(current_line)	
				for option, value in self.fields.items():
					if self.include_file == '1' and option == 'f___n':
						self.record[option] = row
					else:						
						len_ss = len(value)			
						pos_ini = current_line.find(value)
						if pos_ini != -1:
							if len_cl > pos_ini + len_ss:
								new_value = current_line[pos_ini + len_ss:]
								if self.record[option] == '':
									self.record[option] = new_value.strip()				
							if self.search_multiple == '0':
								break
										
			current_file.close()			
			return(self.record)
		else:
			return([])

	def ApplyRules(self, rec):
		'''Apply rules (changes data)
		
		rec -- Python dictionary with field names and data
		
		Returns Python dictionary with field names and data updated
		
		>>> rap1.ApplyRules(Dict)
		
		Functions available for rules:
		
		- rule1 = (fieldname, 'SUBSTR', postion_initial, lenght)
		- rule1 = (fieldname, 'REPLACE', search_string, replace_string)
		- rule1 = (fieldname, 'UPPER')
		- rule1 = (fieldname, 'LOWER')
		- rule1 = (fieldname, 'REVERSE')
		- rule1 = (fieldname, 'REMOVE')
		- rule1 = (fieldname, 'FIELDISDATA')
		- rule1 = (fieldname, 'REMOVEFROM', 'string')
		- rule1 = (fieldname, 'REMOVETO', 'string')
		
		'''			
		if self.number_errors == 0:
			for rule in self.rules:						
				if len(rule) > 1 and len(rule) < 5 and rule[0] in rec:
					field = rule[0]
					operation = rule[1]
					value = rec[field]
					if len(rule) > 2:
						arg1 = rule[2]
					if len(rule) > 3:
						arg2 = rule[3]															
					if operation.upper() == 'SUBSTR' and \
						len(rule) == 4 and type(arg1) is int and \
						type(arg2) is int and \
						len(value) >= arg1 + arg2 - 1:
							rec[field] = value[arg1-1:arg2].strip()							
					elif operation.upper() == 'REPLACE' and \
						len(rule) == 4 and type(arg1) is str and \
						type(arg2) is str:
							rec[field] = value.replace(arg1, arg2, 1)							
					elif operation.upper() == 'UPPER':
						rec[field] = value.upper()						
					elif operation.upper() == 'LOWER':
						rec[field] = value.lower()
					elif operation.upper() == 'REVERSE':
						rec[field] = value[::-1]					
					elif operation.upper() == 'REMOVE':
						rec[field] = ''						
					elif operation.upper() == 'FIELDISDATA':
						rec[field] = field						
					elif operation.upper() == 'REMOVEFROM' and \
						len(rule) == 3 and type(arg1) is str:
							pos_ini = value.find(arg1)
							if pos_ini != -1:
								rec[field] = value[:pos_ini]
					elif operation.upper() == 'REMOVETO' and \
						len(rule) == 3 and type(arg1) is str:
							pos_ini = value.find(arg1)
							if pos_ini != -1:
								rec[field] = value[pos_ini:]																																
		return(rec)

	def ShowError(self):
		'''Show errors
				
		Returns a string with the errors
		
		>>> rap1.ShowError()
		'''			
		if self.number_errors != 0:
			return('Errors: ' + ''.join(map(str, self.errors)))
		else:
			return('')

==> pyraptor-0.1.1.tar.gz: pyraptor-0.1.1/pyraptor/__init__.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# name:        pyraptor.py (Python 3.x).
# description: A tool for searching, extracting, debugging and displaying
#              information from multiple text files.
# author:      Antonio Suárez Jiménez, pherkad13@gmail.com
# date:        26-07-2015
#
#--------------------------------------------------------------------

'''A tool for searching, extracting, debugging and displaying information
from multiple text files.
'''

__author__ = 'Antonio Suárez Jiménez, pherkad13@gmail.com'
__title__= 'pyraptor'
__date__ = '2015-07-26'
__version__ = '0.1.1'
__license__ = 'GNU GPLv3'

import os, configparser, collections

class Pyraptor(object):
	'''Pyraptor for extracting and displaying information from a set of files 
	of the same type; and creating a single CSV file with all the selected
	information.
	
	The information in the files may be in multiple rows:
	
	PC01.txt:
	User=ms123
	Name=Mayra Sanz
	OS=GNU/Linux
	IP=10.226.140.1
	
	But, also, the information may be in several columns. It is possible
	to read data from multiple fields in a single line:
	
	PC01.log:
	User: ms123	    Name: Mayra Sanz
	OS: GNU/Linux   IP: 10.226.140.1		

	Example: data from the following files...

	PC01.txt:
	User=ms123
	Name=Mayra Sanz
	OS=GNU/Linux
	IP=10.226.140.1

	PC02.txt:
	User=lt001
	Name=Luis Toribio
	OS=GNU/Linux
	IP=10.226.140.2

	PC03.txt:
	User=co205
	Name=Clara Osto
	OS=Win
	IP=10.226.140.3

	... You can create a CSV file with the following information:

	User,Name,OS,IP
	MS123,Mayra Sanz,GNU/linux,10.226.140.1
	LT001,Luis Toribio,GNU/linux,10.226.140.2
	CO205,Clara Osto,Win,10.226.140.3

	To achieve this you need to create a template (.rap) with Pyraptor, 
	which is similar to an INI file with the following information:

	users.rap:
	[General]
	description = Get list of users
	extension = txt
	prefix = PC
	output_folder = txt
	input_folder = txt
	output_file = users.csv
	delimiter = ,
	quotechar = "
	include_header = 1
	include_file = 0
	search_multiple = 0

	[Fields]
	user = User=
	name = Name=
	os = OS=
	ip = IP=

	[Rules]
	rule1 = ('user', 'UPPER')

	To create .rap template (If the .rap template exists it is not saved):

	from pyraptor import Pyraptor
	rap1 = Pyraptor()		
	rap1.description = 'Get list of users'
	rap1.extension = 'txt'
	rap1.prefix = 'PC'
	rap1.input_folder = 'txt'
	rap1.output_folder = 'txt'
	rap1.output_file = 'users.csv'
	rap1.delimiter = ','
	rap1.quotechar = '"'
	rap1.include_header = '1'
	rap1.include_file = '0'
	rap1.search_multiple = '0'
	rap1.fields['user'] = 'User='
	rap1.fields['name'] = 'Name='
	rap1.fields['os'] = 'OS='
	rap1.fields['ip'] = 'IP='
	rap1.rules.append(('user', 'UPPER'))
	rap1.Save("users.rap") 	
	del rap1

	Opens template (.rap) and creates (.csv) file from the data read from
	multiple text files:

	from pyraptor import Pyraptor
	import csv
	rap2 = Pyraptor()
	rap2.Open('users.rap')
	if rap2.number_errors == 0:			
		file_csv = open(rap2.output_file, 'w', newline='')
		csv_output = csv.writer(file_csv, 
					delimiter=rap2.delimiter,
					quotechar=rap2.quotechar, 
					quoting=csv.QUOTE_MINIMAL)
		if rap2.include_header == '1':
			fields_list = rap2.BuildHeader()
			print(fields_list)
			csv_output.writerow(fields_list)
							
		for row in rap2.list_files:
			new_record = rap2.ApplyRules(rap2.BuildRow(row))			
			new_record = list(new_record.values())
			print(new_record)
			csv_output.writerow(new_record)
			print('File 'users.csv' saved.')			
		file_csv.close()			
	else:
		print(rap2.ShowError())
	del rap2

	'''	
	app_folder = os.getcwd() + '/'
	def __init__(self):
		'''Initialize a .rap template
		
		Attribute List:
		
		- description: short descripton of .rap template
		- extension: extension of the files to read
		- prefix: files must begin with this string
		- input_folder: folder of files to read
		- output_folder: output folder to save file with result
		- output_file: output filename
		- delimiter: delimiter character
		- quotechar: quote character
		- include_header: '0' or '1'
		- include_file: '0' or '1'
		- search_multiple: '0' or '1'
		- fields: dictionary with fieldnames and search string (read template)
		- record: dictionary with fieldnames and values (read template)
		- rules: list of rules (read template)
		- list_files: list of filenames to read (auto)
		- errors: list of errors (auto)
		- number_errors: number of errors after you open or save a template
		
		>>> from pyraptor import Pyraptor		
		>>> rap1 = Pyraptor()
		>>> rap1.description = 'Description'
		>>> ...				
		'''		
		self.description = ''
		self.extension = 'txt'
		self.prefix = ''
		self.input_folder = Pyraptor.app_folder
		self.output_folder = Pyraptor.app_folder
		self.output_file = 'output.csv'
		self.delimiter = ','
		self.quotechar = '"'
		self.include_header = '1'
		self.include_file = '0'
		self.search_multiple = '0'
		self.fields = collections.OrderedDict({})
		self.record = collections.OrderedDict({})
		self.rules = []
		self.list_files = []
		self.errors = []
		self.number_errors = 0
			
	def Open(self, name):
		'''Open .rap template
		
		name -- .rap template name
		
		>>> rap1.Open('template.rap')
		'''
		self.errors = []		
		if os.path.exists(name):
			config_file = configparser.ConfigParser()
			config_file.read(name)
			if config_file.has_option('General','description'):
				self.description = config_file['General']['description']
			else:
				self.description = ''
			if config_file.has_option('General','extension'):
				self.extension = config_file['General']['extension']
			else:
				self.extension = "txt"
			if config_file.has_option('General','prefix'):
				self.prefix = config_file['General']['prefix']
			else:
				self.prefix = ''
			if config_file.has_option('General','input_folder') and \
				os.path.exists(config_file['General']['input_folder']):
				self.input_folder = config_file['General']['input_folder'] + '/'
			else:
				self.input_folder = Pyraptor.app_folder
				self.errors.append('No source folder. ')
			if config_file.has_option('General','output_folder') and \
				os.path.exists(config_file['General']['output_folder']):
				self.output_folder = config_file['General']['output_folder'] + '/'
			else:
				self.output_folder = Pyraptor.app_folder
				self.errors.append('No target folder. ')
			if config_file.has_option('General','output_file') and \
				config_file['General']['output_file'] != '':
				self.output_file = self.output_folder + config_file['General']['output_file']
			else:
				self.output_file = self.output_folder + "output.csv"				
			if config_file.has_option('General','delimiter') and \
				config_file['General']['delimiter'] != '':
				self.delimiter = config_file['General']['delimiter']
			else:
				self.delimiter = ','
			if config_file.has_option('General','quotechar') and \
				config_file['General']['quotechar'] != '':
				self.quotechar = config_file['General']['quotechar']
			else:
				self.quotechar = '"'
			if config_file.has_option('General','include_header') and \
				config_file['General']['include_header'] in ['0','1'] :
				self.include_header = config_file['General']['include_header']
			else:
				self.include_header = '1'
			if config_file.has_option('General','include_file') and \
				config_file['General']['include_file'] in ['0','1']:
				self.include_file = config_file['General']['include_file']
			else:
				self.include_file = '0'
			if config_file.has_option('General','search_multiple') and \
				config_file['General']['search_multiple'] in ['0','1']:
				self.search_multiple = config_file['General']['search_multiple']
			else:
				self.search_multiple = '0'

			self.fields = collections.OrderedDict({})
			self.record = collections.OrderedDict({})
			if 'Fields' in config_file:
				if self.include_file == '1':
					self.fields['f___n'] = ''
					self.record['f___n'] = ''					
				for option, value in config_file['Fields'].items():
					self.fields[option] = value
					self.record[option] = ''
			
			self.rules = []
			if len(self.fields) > 0:			
				if 'Rules' in config_file:	
					for option, value in config_file['Rules'].items():
						if option.upper().startswith('RULE') and len(option) > 4 and \
							option[4:].strip().isnumeric():
								value = eval(value)
								if type(value) is tuple and len(value) > 1:
									self.rules.append(value)
			else:
				self.errors.append('No field/s. ')

		self.list_files = []			
		if len(self.errors) == 0:
			list_file = os.listdir(self.input_folder)
			for actual_file in list_file:
				if actual_file[-3:].upper() == self.extension.upper():
					if actual_file.startswith(self.prefix):
						self.list_files.append(actual_file)
			
			if len(self.list_files) > 0:
				self.list_files.sort()
			else:
				self.errors.append('No file/s to search. ')
		self.number_errors = len(self.errors)

	def Save(self, name):
		'''It saves .rap template
		
		name -- .rap template name
		
		>>> rap1.Save('template.rap')
		'''
		self.errors = []		
		config_file = configparser.ConfigParser()		
		if os.path.exists(name) == False:
			if len(self.fields) > 0:
				config_file['General'] = {}
				config_file['Fields'] = {}
				config_file['Rules'] = {}
				general = config_file['General']
				fields = config_file['Fields']
				rules = config_file['Rules']
				general['description'] = self.description
				general['extension'] = self.extension
				general['prefix'] = self.prefix
				general['output_folder'] = self.output_folder
				general['input_folder'] = self.input_folder
				general['output_file'] = self.output_file			
				general['delimiter'] = self.delimiter
				general['quotechar'] = self.quotechar
				general['include_header'] = self.include_header
				general['include_file'] = self.include_file
				general['search_multiple'] = self.search_multiple
				for option, value in self.fields.items():
					fields[option] = value
				
				for position in range(len(self.rules)):
					rules['rule' + str(position+1)] = str(self.rules[position])
				
				with open(name, 'w') as output_config:
					config_file.write(output_config)
			else:
				self.errors.append('No field/s. ')
		else:
			self.errors.append('The .rap template exists. ')
		self.number_errors = len(self.errors)
	
	def BuildHeader(self):
		'''Buid header
		
		Returns Python list of field names
		
		>>> rap1.BuildHeader()
		'''
		if self.number_errors == 0:
			return(list(self.fields.keys()))
		else:
			return([])

	def BuildRow(self, row):
		'''Buid row (record)
		
		row -- filename to search
		
		Returns Python dictionary with field names and data
		
		>>> rap1.BuildRow('filename')
		'''
		if self.number_errors == 0:
			for option in self.record.keys():
				self.record[option] = ''		
			current_file = open(self.output_folder + row, 'rb')		
			while True:
				current_line = current_file.readline()
				current_line = current_line.decode("utf-8", "ignore")
				if not current_line: break
				current_line = current_line.rstrip()
				len_cl = len(current_line)	
				for option, value in self.fields.items():
					if self.include_file == '1' and option == 'f___n':
						self.record[option] = row
					else:						
						len_ss = len(value)			
						pos_ini = current_line.find(value)
						if pos_ini != -1:
							if len_cl > pos_ini + len_ss:
								new_value = current_line[pos_ini + len_ss:]
								if self.record[option] == '':
									self.record[option] = new_value.strip()				
							if self.search_multiple == '0':
								break
										
			current_file.close()			
			return(self.record)
		else:
			return([])

	def ApplyRules(self, rec):
		'''Apply rules (changes data)
		
		rec -- Python dictionary with field names and data
		
		Returns Python dictionary with field names and data updated
		
		>>> rap1.ApplyRules(Dict)
		
		Functions available for rules:
		
		- rule1 = (fieldname, 'SUBSTR', postion_initial, lenght)
		- rule1 = (fieldname, 'REPLACE', search_string, replace_string)
		- rule1 = (fieldname, 'UPPER')
		- rule1 = (fieldname, 'LOWER')
		- rule1 = (fieldname, 'REVERSE')
		- rule1 = (fieldname, 'REMOVE')
		- rule1 = (fieldname, 'FIELDISDATA')
		- rule1 = (fieldname, 'REMOVEFROM', 'string')
		- rule1 = (fieldname, 'REMOVETO', 'string')
		
		'''			
		if self.number_errors == 0:
			for rule in self.rules:						
				if len(rule) > 1 and len(rule) < 5 and rule[0] in rec:
					field = rule[0]
					operation = rule[1]
					value = rec[field]
					if len(rule) > 2:
						arg1 = rule[2]
					if len(rule) > 3:
						arg2 = rule[3]															
					if operation.upper() == 'SUBSTR' and \
						len(rule) == 4 and type(arg1) is int and \
						type(arg2) is int and \
						len(value) >= arg1 + arg2 - 1:
							rec[field] = value[arg1-1:arg2].strip()							
					elif operation.upper() == 'REPLACE' and \
						len(rule) == 4 and type(arg1) is str and \
						type(arg2) is str:
							rec[field] = value.replace(arg1, arg2, 1)							
					elif operation.upper() == 'UPPER':
						rec[field] = value.upper()						
					elif operation.upper() == 'LOWER':
						rec[field] = value.lower()
					elif operation.upper() == 'REVERSE':
						rec[field] = value[::-1]					
					elif operation.upper() == 'REMOVE':
						rec[field] = ''						
					elif operation.upper() == 'FIELDISDATA':
						rec[field] = field						
					elif operation.upper() == 'REMOVEFROM' and \
						len(rule) == 3 and type(arg1) is str:
							pos_ini = value.find(arg1)
							if pos_ini != -1:
								rec[field] = value[:pos_ini]
					elif operation.upper() == 'REMOVETO' and \
						len(rule) == 3 and type(arg1) is str:
							pos_ini = value.find(arg1)
							if pos_ini != -1:
								rec[field] = value[pos_ini:]																																
		return(rec)

	def ShowError(self):
		'''Show errors
				
		Returns a string with the errors
		
		>>> rap1.ShowError()
		'''			
		if self.number_errors != 0:
			return('Errors: ' + ''.join(map(str, self.errors)))
		else:
			return('')
